/*
	$Id: zmscanner.h 191 2003-09-15 14:12:16Z crosser $

	$Log: zmscanner.h,v $
	Revision 1.20  2003/09/15 13:32:49  crosser
	make reporting more logical with varpool: just set a variable with
	predefined name and it will go to the SMTP peer!
	
	Revision 1.19  2003/09/14 22:01:13  crosser
	need to initialize hash before readconfig()
	
	Revision 1.18  2003/09/14 21:16:17  crosser
	I hardly beleive it but the new varpool thing seems to work!
	
	Revision 1.17  2003/09/14 20:19:20  crosser
	varpool.h merged into zmscanner.h
	
	Revision 1.16  2003/09/14 11:31:43  crosser
	code cleanup to silence -Wall warnings.  Actually, some of them where bugs.
	Still cannot figure how to initialize `union' without warning
	
	Revision 1.15  2003/09/12 15:22:41  crosser
	break parsing to separate modules (just placeholders for now)
	
	Revision 1.14  2003/09/09 17:40:29  crosser
	wip on dispatcher
	
	Revision 1.13  2003/09/08 12:55:55  crosser
	make static and dynamic plugins syntactically identical
	
	Revision 1.12  2003/09/05 10:10:42  crosser
	make config parameter "gcfg.modconfdir" and access method
	
	Revision 1.11  2003/09/04 21:23:21  crosser
	use strlcpy;
	check_ct: regex matching works
	
	Revision 1.10  2003/09/04 18:03:41  crosser
	changed (and broke) scanning for attributes in the headers
	
	Revision 1.9  2003/09/03 19:41:39  crosser
	work in progress on changed calling conventions
	
	Revision 1.8  2003/09/02 22:20:11  crosser
	make initial stage run time configurable
	
	Revision 1.7  2003/09/02 22:06:05  crosser
	make mattr_t visible to plugins
	define slab_t in preparation to change of calling conventions
	
	Revision 1.6  2003/09/01 16:39:22  crosser
	make DSO actually work
	
	Revision 1.5  2003/09/01 09:39:21  crosser
	add head comments where necessary
	add run time config function
	
	Revision 1.4  2003/09/01 08:42:01  crosser
	make termination func work
	
	Revision 1.3  2003/08/31 21:18:37  crosser
	made plugin framework
	
	Revision 1.2  2003/08/31 12:58:58  crosser
	make basic separation of lib/ plugins
	configure REPLACE_FUNCS in a right way (hopefully)
	
	Revision 1.1  2003/08/31 11:48:46  crosser
	filling inc and lib
	
*/

/*
	WHAT IS IT:
		modularized contentfilter for Zmailer
	COPYRIGHT:
		(c) 2003 Eugene G. Crosser <crosser@average.org>
	LICENSE:
		The same set as apply to Zmailer code
*/

#ifndef ZMSCANNER_H
#define ZMSCANNER_H

size_t
strlcpy(char *dst,const char *src,size_t siz);

typedef struct _slab slab_t;

/* ************************************************************************
	V A R P O O L functions

DOCUMENTATION (kind of):

vp_set_str() creates a *copy* of the string.  Don't allocate yourself
vp_set() does not allocate object (it dows not know how).  Create yourself.
vp_set() *calls destructor* if it cannot insert the object (and returns -1)
	Don't try to destruct it yourself again!
vp_set() destructs any previously existing object with the same name

*/

#define VP_MAXNAME 32
#define VP_ANSWER_STR "zms_answer_string"

typedef struct _varobj {
	char name [VP_MAXNAME];
	void *object;
	void (*destructor)(void *);
} varobj_t;

typedef void * varpool_t;

extern varpool_t
vp_create_vp(void);
extern void
vp_destroy_vp(varpool_t vp);

extern int
vp_set(varpool_t vp,char *name,void *object,void (*destructor)(void *));
extern int
vp_set_int(varpool_t vp,char *name,int value);
extern int
vp_set_str(varpool_t vp,char *name,char *value);

extern void *
vp_get(varpool_t vp,char *name);
extern int
vp_get_int(varpool_t vp,char *name);
extern char *
vp_get_str(varpool_t vp,char *name);

/* ********************************************************************* */

/* O T H E R   Z M A C A N N E R   S T U F F */

struct _slab {
	char *beg;
	char *end;
	void *atr;
};

#define slab_size(x) ((x).end-(x).beg)

typedef enum {t_single, t_multipart, t_message} btype_t;
typedef enum {e_plain, e_qp, e_base64, e_uue} benc_t;
typedef struct _matr {
	btype_t btype;
	benc_t benc;
	slab_t boundary;
	slab_t content_type;
	slab_t name;
	slab_t charset;
} mattr_t;

extern char *
modconfdir(void);

extern void
init_plugins(void);

extern void
term_plugins(void);

extern int
register_plugin(char *stage,char *name,
		int (*init_func)(void **priv),
		void (*term_func)(void **priv),
		int (*scan_func)(char *stage,int depth,
				slab_t data,varpool_t vp,void *priv));

extern int
scandata(char *stage,int depth,slab_t data,varpool_t vp);

#define ZMSCAN_CONTINUE	0
#define ZMSCAN_STOP	-1

#ifdef STATICMODULE
# define ZMS_MODULE(stage,name,init_func,term_func,scan_func) \
	void STATICMODULE (void) \
		{register_plugin(stage,name,init_func,term_func,scan_func);}
# define ZMS_MODINIT_FUNC void STATICMODULE (void)
#else
# define ZMS_MODULE(stage,name,init_func,term_func,scan_func) \
	void zms_mod_init(void) \
		{register_plugin(stage,name,init_func,term_func,scan_func);}
# define ZMS_MODINIT_FUNC void zms_mod_init (void)
#endif

#define ZMS_CLAIM_STAGE(stage,name,init_func,term_func,scan_func) \
	register_plugin(stage,name,init_func,term_func,scan_func)

#endif
